; NAME:
;   buildgalaxy.pro (Galaxy Builder)
;   
; PURPOSE:
;   Creates a galaxy SLD (or SED) from a list of fundamental properties.
;
; CALLING SEQUENCE:
;   buildgalaxy, freq, Lnu, [properties]
;   buildgalspec, freq, Fnu, z=z, [properties]
;
; INPUTS:
;   see buildgalaxy.pro
;
; OUTPUTS:
;    freq : Frequency of the SED.
;    wav  : Wavelengths of the SED.
;    fnu  : Flux density at this frequency (uJy)
;    Lbol : output total luminosity (to fix as an input, use Ltot)
;    Lir  : output luminosity from dust reradiation
;    hist : structure showing star-formation history and other time-dependent parameters
;    lines : list of lines producing nebular emission (note: strong UV/optical lines only)
;    linefluxes : total fluxes of these lines



pro buildgalspec, freq, fnu, z=z, $
                 pinfo=pinfo, $
                 sfr=sfr, tav=tav, age=age, model=model, texp=texp, tsmooth=tsmooth, metallicity=metallicity, mass=mass, $
                 embedfrac=embedfrac, av=av, embedav=embedav, dustlaw=dustlaw, rv=rv, $
                 Tdust=Tdust, gasmetal=gasmetal, q=q, $
                 binning=binning, resolution=resolution, $
                 Ltot=Ltot, Lbol=Lbol, Lir=Lir, hist=hist, $
                 lines=lines, linefluxes=linefluxes, sethist=sethist, nonebular=nonebular, $
                 wav=wav

   ; Calculate the OBSERVED spectrum in OBSERVED frequency/wavelength.
   ; wav in Angstrom, fnu in uJy, freq in GHz
   ; wav is an output

   if n_elements(pinfo) gt 0 then begin
        z = pinfo.z
        sfr = pinfo.sfr
        tav = pinfo.tav
        age = [pinfo.age, pinfo.age2]
        texp = pinfo.tau
        tsmooth= pinfo.tsmooth
        metallicity = pinfo.metal
        mass = pinfo.mass
        embedfrac = pinfo.fembed
        av = pinfo.av
        embedav = pinfo.avembed
        dustlaw = pinfo.dustlaw
        rv = pinfo.rv
        Tdust = pinfo.Tdust
        gasmetal = pinfo.gasmetal
        q = pinfo.q
        lbol = pinfo.L
   endif

   buildgalaxy, freq, Lnu, $
                 sfr=sfr, tav=tav, age=age, model=model, texp=texp, tsmooth=tsmooth, metallicity=metallicity, mass=mass, $
                 embedfrac=embedfrac, av=av, embedav=embedav, dustlaw=dustlaw, rv=rv, $
                 Tdust=Tdust, gasmetal=gasmetal, q=q, $
                 binning=binning, resolution=resolution, $
                 Ltot=Ltot, Lbol=Lbol, Lir=Lir, hist=hist, $
                 lines=lines, lineluminosity=lineluminosity, sethist=sethist, nonebular=nonebular

   dLMpc = lumdist(z, H0=72, Omega_M=0.3, Lambda0=0.7, /silent)
   dLcgs =   3.086d+24*dLMpc 

   fnu = Lnu / (4*!pi*dLcgs^2)     ; first, distance fade
   freq = freq / (1+z)                ; redshift
   fnu = fnu * (1+z)                  ; redshift-related flux normalization
   wav = 3e+9/freq
   fnu = fnu * igm_atten(wav, z)        ; IGM attenuation
 
   fnu = fnu * 1d+29                 ;cgs -> uJy
   

   if keyword_set(nonebular) eq 0 then linefluxes = lineluminosity / (4*!pi*dLcgs^2)

end

