function synphot, w, flux, filter, xunit=xunit, yunit=yunit, outunit=outunit

  ;# Calculate synthetic photometry of a spectrum in a specified filter.

  ;# Standard filters can be chosen with "filter".  Currently available filters:
  ;#    B, V, R, I, u, g, r, i z, J, H, K, Kp, Ks, F110W, F140W, F160W, F606W, F814W, W1, W2, W3, W4
  ;# Nonstandard filters can be specified with "filename".

  ;# Output is the flux in microJy.
  
  ;# The transmission curves are loaded from disk, interpolated to the SED, and cached in memory for
  ;#   reuse.  

  ;# default:  w: wavelength [in Angstroms],  flux: flambda [in erg/cm^2/s/Ang]
  ;# alternative xunits:  nm, (Angstrom), Hz, GHz
  ;# alternative yunits:  flambda_cgs, uJy, Jy, fnu [in cgs]
  ;# alternative outunits:  none currently available

  common transmissionblock, trans, itrans

  printinterpnote = 0

  if n_elements(w) ne n_elements(flux) then begin
     print, 'x and y arrays must be same length!'
     return, NaN
  endif
  if n_elements(w) lt 10 then begin
     print, 'WARNING: remarkably few elements in input arrays.'
  endif

  if n_elements(xunit) eq 0 then xunit = 'Ang'
  if n_elements(yunit) eq 0 then yunit = 'flambda'
  if xunit eq 'ang' then xunit = 'Ang'
  if yunit eq 'flambda/ang' then yunit = 'flambda_ang'

  n = n_elements(w)
  nf = n_elements(flux)
  nfilt = n_elements(filter)

  ;# load the raw transmission data from the .sav file
  if n_elements(trans) eq 0 then trans = gettrans()


  ;# check if there are any nonstandard filters requested (not yet active)


  ;# do x unit conversion if necessary
  if xunit eq 'Ang' then wav = w        ;# wav is always in Ang.
  if xunit eq 'nm'  then wav = w*10.
  if xunit eq 'um'  then wav = w*1e+4
  if xunit eq 'cm'  then wav = w*1e+7
  if xunit eq 'm'   then wav = w*1e+10
  if n_elements(wav) gt 0 then freq = 3e+18/wav ;# Hz
  
  if xunit eq 'Hz'  then freq = w
  if xunit eq 'GHz' then freq = 1e+9 * w  ;# freq is always in Hz.
  if n_elements(wav) eq 0 then wav = 3e+18/freq


  ;# if there has been a change in the length of the array we must reconstitute the structure.
  if n_elements(itrans) eq 0 then $
     itrans = replicate({filt:'', file:'', wav:fltarr(n), trans:fltarr(n)},n_elements(trans))
  if n_elements(itrans[0].wav) ne n then $
     itrans = replicate({filt:'', file:'', wav:fltarr(n), trans:fltarr(n)},n_elements(trans))
  if itrans[0].filt eq '' then itrans.filt = trans.filt

  ;# interpolate the transmission curves for filters whose wavs do not match the input wav.
  ninterp = 0
  for f = 0, nfilt-1 do begin ;# only looking at requested filters
     t = where(trans.filt eq filter[f], ct)
     if ct eq 0 then begin
        print, 'Filter ', filter[f] ,' not found in database!!'  ;# eventually move this check elsewhere
        continue
     endif
     ; total(float(trans[t].wav) ne float(wav)) gt 0
     isalmostequal = abs(float(itrans[t].wav)-float(wav)) gt 0.0001*float(wav)
     if total(isalmostequal ne 0) or total(itrans[t].wav) le 0 then begin
        if printinterpnote then begin
           if ninterp eq 0 then  print, 'Interpolating filter(s) ', format='($,A)'
           if ninterp ge 1 then print, ',', format='($,A)'
           print, filter[f], format='($,A)'
        endif
        ninterp += 1

        itrans[t].wav = wav
        nz = where(trans[t].inwav ne 0)
        itrans[t].trans = interpol(trans[t].intrans[nz],trans[t].inwav[nz], wav) 
        ;# set transmission curve to 0 below minimum and above maximum wavelength value
        belowmin = where(wav lt min(trans[t].inwav[nz]), ct)
        if ct gt 0 then itrans[t].trans[belowmin] = 0. $ 
                   else print, string(10B), 'WARNING: transmission extends beyond spectral range!'
        abovemax = where(wav gt max(trans[t].inwav[nz]), ct)
        if ct gt 0 then itrans[t].trans[abovemax] = 0.$ 
                   else print, string(10B), 'WARNING: transmission extends beyond spectral range!'
        ;plot, trans[t].inwav[nz], trans[t].intrans[nz], linestyle=1
        ;oplot, itrans[t].wav, itrans[t].trans
     endif
  endfor
  if printinterpnote then if ninterp gt 0 then print, ' to input wavelengths.'

  ;if yunit eq 'flambda' then begin
    if yunit eq 'flambda' or yunit eq 'flambda_ang' then flambda = flux*1e8     ;# erg/cm^2/s/Ang -->
    if yunit eq 'flambda_cgs' then flambda = flux  ;# erg/cm^2/s/cm
  ;endif else if yunit eq 'fnu' or yunit eq 'uJy' or yunit eq 'Jy' then begin
    if yunit eq 'fnu' then fnu = flux*1d+29  ;# erg/cm^2/s/Hz
    if yunit eq 'uJy' then fnu = flux
    if yunit eq 'Jy'  then fnu = flux*1d+6
  ;endif

  ;# Done with the hard part!  Now we actually do the synthetic photometry.

  outflux = fltarr(nfilt)


  ;# flux given as f_lambda
  if n_elements(flambda) gt 0 then begin
     ;# could cache this
     dwav = abs(shift(wav,1) - shift(wav,-1))/2.
     dwav[0] = abs(wav[1]-wav[0])
     dwav[n-1] = abs(wav[n-1]-wav[n-2])
     bad = where(finite(dwav) eq 0, ct)
     if ct gt 0 then dwav[bad] = 0

     for f = 0, nfilt-1 do begin

        ;# f_lambda = f_nu * c / lambda^2
        ;#          = (1e-29 erg/s/cm^2/Hz) * 3e+10 cm/s / (Ang * 1e-8 cm)^2
        ;#          = 3e-3 erg/s/cm^2/cm
        flatsed = 2.99792458d-3/(wav^2) ; sed with 1 uJy everywhere
        bad = where(finite(flatsed) eq 0, ct) ; a wav=0 point will mess things up
        if ct gt 0 then flatsed[bad] = 0

        t = where(itrans.filt eq filter[f], ct)
        realflux = total(flambda * dwav * itrans[t].trans)
        flatflux = total(flatsed * dwav * itrans[t].trans)
        outflux[f] = realflux/flatflux

    endfor
  endif
  ;# flux given as f_nu
  if n_elements(fnu) gt 0 then begin
    ;# could cache this
    dfreq = abs(shift(freq,1) - shift(freq,-1))/2.
    dfreq[0] =  abs(freq[1]-freq[0])
    dfreq[n-1] = abs(freq[n-1]-freq[n-2])
    bad = where(finite(dfreq) eq 0, ct)
    if ct gt 0 then dfreq[bad] = 0

    for f = 0, nfilt-1 do begin
        flatsed = 1.+fltarr(n_elements(freq)) ; sed with 1 uJy everywhere
        bad = where(finite(flatsed) eq 0, ct) 
        if ct gt 0 then flatsed[bad] = 0

        t = where(itrans.filt eq filter[f], ct)
        realflux = total(fnu * dfreq * itrans[t].trans)
        flatflux = total(flatsed * dfreq * itrans[t].trans)
        outflux[f] = realflux/flatflux

    endfor
  endif

  if nfilt eq 1 then outflux = outflux[0]

  return, outflux

end

